@@ -30,7 +30,7 @@ class Camera(ABC):
                  name=None,
                  camtype=None,
                  rho=10e-6,
-                 imagesize=(500, 500),
+                 imagesize=(1024, 1024),
                  pose=None):
         """
         Create instance of a Camera class
@@ -157,28 +157,15 @@ class Camera(ABC):
     @property
     def rho(self):
         """
-        Get pixel size: horizontal value
+        Get pixel dimensions
 
         :return: horizontal pixel size
         :rtype: float
 
-        :seealso: :func:`rhov`, :func:`rho`
+        :seealso: :func:`rhou`, :func:`rhov`
         """
 
-        return (self._rhov, self._rhov)
-
-    @property
-    def rho(self):
-        """
-        Get pixel size: horizontal value
-
-        :return: horizontal pixel size
-        :rtype: float
-
-        :seealso: :func:`rhov`, :func:`rho`
-        """
-        return (self._rhov, self._rhov)
-
+        return (self._rhou, self._rhov)
 
     @property
     def image(self):
@@ -223,6 +210,7 @@ class Camera(ABC):
             ax.set_xlabel('u (pixels)')
             ax.set_ylabel('v (pixels)')
             ax.set_title(self.name)
+            ax.set_facecolor('lightyellow')
             ax.figure.canvas.set_window_title('Machine Vision Toolbox for Python')
 
         # TODO figure out axes ticks, etc
@@ -230,7 +218,7 @@ class Camera(ABC):
         self._ax = ax
         return fig, ax  # likely this return is not necessary
 
-    def plot(self, p=None, marker='or', markersize=10, **kwargs):
+    def plot(self, p=None, marker='or', markersize=6, **kwargs):
         """
         Plot points on image plane
         If 3D points, then 3D world points
@@ -451,7 +439,6 @@ class CentralCamera(Camera):
 
     def __init__(self,
                  f=8*1e-3,
-                 rho=10e-6,
                  pp=None,
                  **kwargs):
         """
@@ -459,7 +446,6 @@ class CentralCamera(Camera):
         """
 
         super().__init__(**kwargs)
-
         # TODO some of this logic to f and pp setters
         self.f = f
 
@@ -531,7 +517,6 @@ class CentralCamera(Camera):
 
             x[2,x[2,:]<0] = np.nan  # points behind the camera are set to NaN
 
-
             x = base.h2e(x)
 
             # if self._distortion is not None:
@@ -874,7 +859,7 @@ class CentralCamera(Camera):
 
         u, s, v = scipy.linalg.svd(C)
 
-        t = v[:,3]
+        t = v[3,:]  # not svd returns v transpose
         t = t / t[3]
         t = t[0:3]
 
@@ -938,6 +923,68 @@ class CentralCamera(Camera):
 
         return C
 
+    def visjac_p(self, uv, Z):
+        '''
+        Image Jacobian for point features (interaction matrix)
+        
+        Returns a 2Nx6 matrix of stacked Jacobians, one per image-plane point.
+        uv is a 2xN matrix of image plane points
+        Z  is the depth of the corresponding world points. Can be scalar, same distance to every
+        point, or a vector or list of length N.
+        
+        References:
+        * A tutorial on Visual Servo Control", Hutchinson, Hager & Corke, 
+            IEEE Trans. R&A, Vol 12(5), Oct, 1996, pp 651-670.
+        * Robotics, Vision & Control, Corke, Springer 2017, Chap 15.
+        '''
+        uv = base.getmatrix(uv, (2, None))
+
+        Z = base.getvector(Z)
+        if len(Z) == 1:
+            Z = np.repeat(Z, uv.shape[1])
+        elif len(Z) != uv.shape[1]:
+                raise ValueError('Z must be a scalar or have same number of columns as uv')
+            
+        L = np.empty((0, 6))  # empty matrix
+
+        K = self.K
+        Kinv = np.linalg.inv(K)
+        
+        for z, p in zip(Z, uv.T):  # iterate over each column (point)
+
+            # convert to normalized image-plane coordinates
+            xy = Kinv @ base.e2h(p)
+            x = xy[0,0]
+            y = xy[1,0]
+
+            # 2x6 Jacobian for this point
+            Lp = K[:2,:2] @ np.array(
+                [ [-1/z,  0,     x/z, x * y,      -(1 + x**2), y],
+                  [ 0,   -1/z,   y/z, (1 + y**2), -x*y,       -x] ])
+
+            # stack them vertically
+            L = np.vstack([L, Lp])
+
+        return L
+
+    def flowfield(self, vel, Z=2):
+        vel = base.getvector(vel, 6)
+
+        u = np.arange(0, self.nu, 50)
+        v = np.arange(0, self.nv, 50)
+        [U,V] = np.meshgrid(u, v, indexing='ij')
+        du = np.empty(shape=U.shape)
+        dv = np.empty(shape=U.shape)
+        for r in range(U.shape[0]):                      
+            for c in range(U.shape[1]):
+                J = self.visjac_p((U[r,c], V[r,c]), Z )            
+                ud, vd =  J @ vel
+                du[r,c] = ud
+                dv[r,c] = -vd
+
+        self.plotcreate()
+        plt.quiver(U, V, du, dv, 0.4)
+        plt.show(block=True)
 
 # ----------------------------------------------------------------------------#
 class CameraVisualizer:
@@ -1009,14 +1056,19 @@ class CameraVisualizer:
 
 if __name__ == "__main__":
 
-    c = CentralCamera()
-    print(c)
-    c.T = SE3([0.1, 0.2, 0.3])
-    print(c.T)
+    cam = CentralCamera()
+    print(cam)
+    # cam.pose = SE3([0.1, 0.2, 0.3])
+    print(cam.pose)
     # fig, ax = c.plot_camera(frustum=True)
     # plt.show()
+    np.set_printoptions(linewidth=120, formatter={'float': lambda x: f"{x:8.4g}" if abs(x) > 1e-10 else f"{0:8.4g}"})
 
-    print(c.project([1,2,-3]))
+
+    print(cam.project([1,2,3]))
+
+    print(cam.visjac_p((300,300), 1))
+    cam.flowfield([0,0,0, 0,0,1])
     # # fundamental matrix
     # # create +8 world points (20 in this case)
     # nx, ny = (4, 5)
