@@ -13,7 +13,8 @@ class ImageProcessingKernelMixin:
     Image processing kernel operations on the Image class
     """
 
-    def kgauss(self, sigma, hw=None):
+    @staticmethod
+    def kgauss(sigma, hw=None):
         """
         Gaussian kernel
 
@@ -33,7 +34,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -53,7 +54,8 @@ class ImageProcessingKernelMixin:
         # an approximation
         return m / np.sum(m)
 
-    def klaplace(self):
+    @staticmethod
+    def klaplace():
         r"""
         Laplacian kernel
 
@@ -72,7 +74,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -82,7 +84,8 @@ class ImageProcessingKernelMixin:
                          [1, -4, 1],
                          [0, 1, 0]])
 
-    def ksobel(self):
+    @staticmethod
+    def ksobel():
         r"""
         Sobel edge detector
 
@@ -108,7 +111,8 @@ class ImageProcessingKernelMixin:
                          [2, 0, -2],
                          [1, 0, -1]]) / 8.0
 
-    def kdog(self, sigma1, sigma2=None, hw=None):
+    @staticmethod
+    def kdog(sigma1, sigma2=None, hw=None):
         """
         Difference of Gaussians kernel
 
@@ -135,7 +139,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -161,7 +165,8 @@ class ImageProcessingKernelMixin:
 
         return m2 - m1
 
-    def klog(self, sigma, hw=None):
+    @staticmethod
+    def klog(sigma, hw=None):
         """
         Laplacian of Gaussian kernel
 
@@ -181,7 +186,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         """
 
@@ -194,7 +199,8 @@ class ImageProcessingKernelMixin:
             ((np.power(x, 2) + np.power(y, 2)) / (2.0 * sigma ** 2) - 1) * \
             np.exp(-(np.power(x, 2) + np.power(y, 2)) / (2.0 * sigma ** 2))
 
-    def kdgauss(self, sigma, hw=None):
+    @staticmethod
+    def kdgauss(sigma, hw=None):
         """
         Derivative of Gaussian kernel
 
@@ -215,7 +221,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -232,7 +238,8 @@ class ImageProcessingKernelMixin:
         return -x / sigma ** 2 / (2.0 * np.pi) * \
             np.exp(-np.power(x, 2) + np.power(y, 2) / 2.0 / sigma ** 2)
 
-    def kcircle(self, r, hw=None):
+    @staticmethod
+    def kcircle(r, hw=None):
         """
         Circular structuring element
 
@@ -252,7 +259,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -279,10 +286,9 @@ class ImageProcessingKernelMixin:
         if not argcheck.isscalar(r):
             s = self.kcircle(rmax, w) - self.kcircle(rmin, w)
         else:
-            x, y = self.meshgrid(s)
-            x = x - c
-            y = y - c
-            ll = np.where(np.round((x ** 2 + y ** 2 - r ** 2) <= 0))
+            x = np.arange(w) - c
+            X, Y = np.meshgrid(x, x)
+            ll = np.where(np.round((X ** 2 + Y ** 2 - r ** 2) <= 0))
             s[ll] = 1
         return s
 
@@ -315,7 +321,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -403,7 +409,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         """
 
@@ -435,7 +441,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         """
 
@@ -466,7 +472,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         """
         if not np.all(self.shape == im2.shape):
@@ -496,7 +502,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -528,7 +534,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -557,7 +563,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -602,7 +608,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -681,7 +687,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -735,7 +741,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -812,7 +818,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -910,7 +916,7 @@ class ImageProcessingKernelMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -963,3 +969,8 @@ class ImageProcessingKernelMixin:
 if __name__ == '__main__':
 
     print('ImageProcessingKernel.py')
+    from machinevisiontoolbox import Image
+
+    image = Image('monalisa.png', grey=True)
+    blur = image.smooth(3, 5)
+    blur.disp(block=True)
\ No newline at end of file
