@@ -11,7 +11,7 @@ class ImageProcessingMorphMixin:
     Image processing morphological operations on the Image class
     """
 
-    def getse(self, se):
+    def _getse(self, se):
         """
         Get structuring element
 
@@ -23,7 +23,11 @@ class ImageProcessingMorphMixin:
         - ``IM.getse(se)`` converts matrix ``se`` into a uint8 numpy array for
           opencv, which only accepts kernels of type CV_8U
         """
-        return np.array(se).astype(np.uint8)
+        se = np.array(se).astype(np.uint8)
+        if se.min() < 0:
+            raise ValueError('cannot convert array with negative values to a structuring element')
+
+        return se
 
     def erode(self, se, n=1, opt='replicate', **kwargs):
         """
@@ -61,7 +65,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -70,7 +74,7 @@ class ImageProcessingMorphMixin:
         """
 
         # check if valid input:
-        se = self.getse(se)
+        se = self._getse(se)
         # TODO check if se is valid (odd number and less than im.shape)
         # consider cv.getStructuringElement?
         # eg, se = cv.getStructuringElement(cv.MORPH_RECT, (3,3))
@@ -136,7 +140,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -145,7 +149,7 @@ class ImageProcessingMorphMixin:
         """
 
         # check if valid input:
-        se = self.getse(se)
+        se = self._getse(se)
 
         if not isinstance(n, int):
             n = int(n)
@@ -222,7 +226,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -232,7 +236,7 @@ class ImageProcessingMorphMixin:
 
         # check if valid input:
         # se = cv.getStructuringElement(cv.MORPH_RECT, (3,3))
-        se = self.getse(se)
+        se = self._getse(se)
 
         # TODO check if se is valid (odd number and less than im.shape),
         # can also be a scalar
@@ -300,7 +304,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -443,7 +447,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -499,7 +503,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -531,7 +535,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         :references:
 
@@ -606,7 +610,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -659,7 +663,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -752,7 +756,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         """
 
@@ -785,7 +789,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. notes::
 
@@ -824,7 +828,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. notes::
 
@@ -866,7 +870,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
@@ -895,7 +899,7 @@ class ImageProcessingMorphMixin:
 
         Example:
 
-        .. autorun:: pycon
+        .. runblock:: pycon
 
         .. note::
 
