@@ -10,7 +10,7 @@ import cv2 as cv
 from spatialmath import base
 from ansitable import ANSITable, Column
 from machinevisiontoolbox.IImage import IImage
-from machinevisiontoolbox import color_bgr, Image
+from machinevisiontoolbox.base import color_bgr, plot_box, plot_labelbox, plot_point
 
 # NOTE, might be better to use a matplotlib color cycler
 import random as rng
@@ -228,7 +228,6 @@ class Blob:
                                  closed=closed)
                  for i, c in enumerate(self._contours)]
         bbox = [cv.boundingRect(cpoly[i]) for i in range(len(cpoly))]
-        print('bbox', bbox)
         return bbox
 
     def _computeequivalentellipse(self):
@@ -383,57 +382,59 @@ class Blob:
                 children[i] = [-1]
         return children
 
-    def plot_box(self, image=None, color=None, label=True, 
-            font=cv.FONT_HERSHEY_SIMPLEX, fontsize=0.9, fontthickness=2):
+    def plot_box(self, **kwargs):
+        """
+        Plot a bounding box for the blob using matplotlib
 
-        if isinstance(color, str):
-            color = color_bgr(color)
+        :param kwargs: arguments passed to ``plot_box``
 
-        if isinstance(image, plt.Axes):
-            for blob in self:
-                x = blob.bbox[0]
-                y = blob.bbox[1]
-                print(x, y)
-                print([x[0], x[0], x[1], x[1], x[0]])
-                print([y[0], y[0], y[1], y[1], y[0]])
-                image.plot([x[0], x[1], x[1], x[0], x[0]], 
-                         [y[0], y[0], y[1], y[1], y[0]], color=color)
-            plt.draw()
+        Plot a bounding box for every blob described by this object.
+
+        :seealso: :func:`~graphics.plot_box`
+        """
+
+        for blob in self:
+            plot_box(bbox=blob.bbox, **kwargs)
+
+
+    def plot_labelbox(self, **kwargs):
+        """
+        Plot a labelled bounding box for the blob using matplotlib
+
+        :param kwargs: arguments passed to ``plot_labelbox``
+
+        Plot a labelled bounding box for every blob described by this object.
+        The blobs are labeled by their blob index.
+
+        :seealso: :func:`~graphics.plot_labelbox`
+        """
+
+        for i, blob in enumerate(self):
+            plot_labelbox(text=f"{i}", bbox=blob.bbox, **kwargs)
+
+    def plot_centroid(self, label=False, **kwargs):
+        """
+        Draw the centroid of the blob using matplotlib
+
+        :param label: add a sequential numeric label to each point, defaults to False
+        :type label: bool
+        :param kwargs: other arguments passed to ``plot_point``
+
+        If no marker style is given then it will be an overlaid "o" and "x"
+        in blue.
+
+        :seealso: :func:`plot_point`
+        """
+        if label:
+            text = f"{i}"
         else:
-            if isinstance(image, IImage):
-                image = image.image
-            for i, blob in enumerate(self):
-                bb = blob.bbox
-                h = 5
-                # put a box around the blob
-                cv.rectangle(image, tuple(bb[:,0]), tuple(bb[:,1]), color)
-
-                if label:
-                    # get size of text:  ((w,h), baseline)
-                    twh = cv.getTextSize(f"{i}", font, fontsize, fontthickness)
-
-                    # filled rectangle
-                    cv.rectangle(image, 
-                        (blob.umin, blob.vmin - twh[0][1] - twh[1] - 2* h),
-                        (blob.umin + twh[0][1], blob.vmin),
-                        color, -1)
-
-                    # add black text in rectangle
-                    # give bottom left corner
-                    cv.putText(image, f"{i}", 
-                        (blob.umin, blob.vmin - twh[1] - h), 
-                        font, fontsize, (0, 0, 0), fontthickness)
-
-
-    def plot_centroid(self, image=None, text=None, color=None):
-        if isinstance(image, plt.Axes):
-            for blob in self:
-                image.text(blob.uc, blob.vc, text, color=color)
-            plt.draw()
-        else:
-            for blob in self:
-                bb = blob.bbox
-                cv.rectangle(image, bb[0], bb[1], color)
+            text = None
+        
+        if 'marker' not in kwargs:
+            kwargs['marker'] = ['bx', 'bo']
+            kwargs['fillstyle'] = 'none'
+        for i, blob in enumerate(self):
+            plot_point(pos=blob.centroid, text=text, **kwargs)
 
     def drawBlobs(self,
                   image,
@@ -442,6 +443,24 @@ class Blob:
                   color=None,
                   contourthickness=cv.FILLED,
                   textthickness=2):
+        """
+        Draw the blob contour
+
+        :param image: [description]
+        :type image: [type]
+        :param drawing: [description], defaults to None
+        :type drawing: [type], optional
+        :param icont: [description], defaults to None
+        :type icont: [type], optional
+        :param color: [description], defaults to None
+        :type color: [type], optional
+        :param contourthickness: [description], defaults to cv.FILLED
+        :type contourthickness: [type], optional
+        :param textthickness: [description], defaults to 2
+        :type textthickness: int, optional
+        :return: [description]
+        :rtype: [type]
+        """
         # draw contours of blobs
         # contours - the contour list
         # icont - the index of the contour(s) to plot
@@ -534,14 +553,32 @@ class Blob:
 
     @property
     def area(self):
+        """
+        Area of the blob
+
+        :return: area in pixels
+        :rtype: int
+        """
         return self._area
 
     @property
     def uc(self):
+        """
+        u-coordinate of the blob centroid
+
+        :return: u-coordinate (horizontal)
+        :rtype: float
+        """
         return self.u
 
     @property
     def vc(self):
+        """
+        v-coordinate of the blob centroid
+
+        :return: v-coordinate (vertical)
+        :rtype: float
+        """
         return self.v
 
     # TODO probably should stick with u,v properties to be consistent with
@@ -556,18 +593,48 @@ class Blob:
 
     @property
     def a(self):
+        """
+        Radius of equivalent ellipse
+
+        :return: largest ellipse radius
+        :rtype: float
+
+        :seealso: func:`b`, :func:`aspect`
+        """
         return self._a
 
     @property
     def b(self):
+        """
+        Radius of equivalent ellipse
+
+        :return: smallest ellipse radius
+        :rtype: float
+
+        :seealso: func:`a`, :func:`aspect`
+        """
         return self._b
 
     @property
     def aspect(self):
+        r"""
+        Blob aspect ratio
+
+        :return: ratio of equivalent ellipse axes, :math:`<= 1`
+        :rtype: float
+
+        :seealso: func:`a`, :func:`b`
+        """
         return self._aspect
 
     @property
     def orientation(self):
+        """
+        Blob orientation
+
+        :return: Orientation of equivalent ellipse (in radians)
+        :rtype: float
+        """
         return self._orientation
 
     @property
@@ -575,10 +642,12 @@ class Blob:
         """
         Bounding box
 
-        :return: [description]
-        :rtype: tuple of tuples
+        :return: bounding
+        :rtype: ndarray(2,2)
 
-        The bounding box is nested tuples ``(u1, u2), (v1, v2)``.
+        The bounding box is a 2x2 matrix  [u1, u2; v1, v2].  The rows are the
+        u- and v-axis extent respectively.  The columns are the bottom-left
+        and top-right corners of the bounding box.
         """
         return np.array([
             [self._umin, self._umax], 
@@ -587,47 +656,135 @@ class Blob:
 
     @property
     def umin(self):
+        """
+        Minimum u-axis extent
+
+        :return: maximum u-coordinate of the blob
+        :rtype: int
+        """
         return self._umin
 
     @property
     def umax(self):
+        """
+        Maximum u-axis extent
+
+        :return: maximum u-coordinate of the blob
+        :rtype: int
+        """
         return self._umax
 
     @property
     def vmax(self):
+        """
+        Minimum b-axis extent
+
+        :return: maximum v-coordinate of the blob
+        :rtype: int
+        """
         return self._vmax
 
     @property
     def vmin(self):
+        """
+        Maximum u-axis extent
+
+        :return: maximum v-coordinate of the blob
+        :rtype: int
+        """
         return self._vmin
 
     @property
     def bboxarea(self):
+        """
+        Area of the bounding box
+
+        :return: area of the bounding box in pixels
+        :rtype: int
+
+        .. note:: The bounding box has vertical and horizontal edges.
+        """
         return [(b._umax - b._umin) * (b._vmax - b._vmin) for b in self]
 
     @property
     def centroid(self):
+        """
+        Centroid of blob
+
+        :return: centroid of the blob
+        :rtype: 2-tuple
+
+        :seealso:  :func:`uc`, :func:`vc`
+        """
         return (self._uc, self.vc)
         # TODO maybe ind for centroid: b.centroid[0]?
 
     @property
     def perimeter(self):
+        """
+        Perimeter of the blob
+
+        :return: perimeter in pixels
+        :rtype: float
+
+
+        """
         return self._perimeter
 
     @property
     def touch(self):
+        """
+        Blob edge touch status
+
+        :return: blob touches the edge of the image
+        :rtype: bool
+        """
         return self._touch
 
     @property
     def circularity(self):
+        r"""
+        Blob circularity
+
+        :return: circularity
+        :rtype: float
+
+        Computed as :math:`\rho = \frac{A}{4 \pi p^2}`.  Is one for a circular
+        blob and < 1 for all other shapes, approaching zero for a line.
+
+        .. note::  Apply Kulpa's correction factor to account for edge
+            discretization:
+
+            - Area and perimeter measurement of blobs in discrete binary pictures.
+              Z.Kulpa. Comput. Graph. Image Process., 6:434-451, 1977.
+    
+            - Methods to Estimate Areas and Perimeters of Blob-like Objects: a
+              Comparison. Proc. IAPR Workshop on Machine Vision Applications.,
+              December 13-15, 1994, Kawasaki, Japan
+              L. Yang, F. Albregtsen, T. Loennestad, P. Groettum
+        """
         return self._circularity
 
     @property
     def parent(self):
+        """
+        Parent blob
+
+        :return: index of this blob's parent
+        :rtype: int
+
+        A parent of -1 is the image background.
+        """
         return self._parent
 
     @property
     def children(self):
+        """
+        Child blobs
+
+        :return: list of indices of this blob's children
+        :rtype: list of int
+        """
         return self._children
 
     def printBlobs(self):
@@ -655,48 +812,81 @@ class BlobFeaturesMixin:
     """
 
     def blobs(self, **kwargs):
+        """
+        Compute blobs in image
+
+        :return: blobs
+        :rtype: Blob
+
+        ``image.blobs()`` is a ``Blob`` object that contains information about
+        all the blobs in the image.  It behaves like a list object so it can
+        be indexed and sliced.
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from machinevisiontoolbox import Image
+            >>> im = Image('shark2.png')
+            >>> blobs = im.blobs()
+            >>> print(blobs)
+
+        
+        """
         return Blob(self, **kwargs)
 
 
 if __name__ == "__main__":
 
-    # read image
     from machinevisiontoolbox import Image
-    im = Image(cv.imread('images/multiblobs.png', cv.IMREAD_GRAYSCALE))
+    import matplotlib.pyplot as plt
 
-    # call Blobs class
-    b = Blob(image=im)
+    im = Image('shark2.png')
+    blobs = im.blobs()
+    print(blobs)
+    im.disp()
+    blobs.plot_labelbox(color='yellow')
+    blobs.plot_centroid()
+    plt.show()
 
-    # plot image
-    # plot centroids of blobs
-    # label relevant centroids for the labelled blobs
-    # import random as rng  # for random colors of blobs
-    rng.seed(53467)
 
-    drawing = np.zeros((im.shape[0], im.shape[1], 3), dtype=np.uint8)
-    colors = [None]*len(b)
-    icont = [None]*len(b)
-    for i in range(len(b)):
-        icont[i] = i
-        colors[i] = (rng.randint(0, 256), rng.randint(
-            0, 256), rng.randint(0, 256))
+    # # read image
+    # from machinevisiontoolbox import Image
+    # im = Image(cv.imread('images/multiblobs.png', cv.IMREAD_GRAYSCALE))
 
-        cv.rectangle(drawing, (b[i].umin, b[i].vmin), (b[i].umax, b[i].vmax),
-                     colors[i], thickness=2)
-        # cv.putText(drawing, str(i), (int(b[i].uc), int(b[i].vc)),
-        #           fontFace=cv.FONT_HERSHEY_SIMPLEX, fontScale=1,
-        #           color=colors, thickness=2)
+    # # call Blobs class
+    # b = Blob(image=im)
 
-    drawing = b.drawBlobs(im, drawing, icont, colors,
-                          contourthickness=cv.FILLED)
-    # mvt.idisp(drawing)
+    # # plot image
+    # # plot centroids of blobs
+    # # label relevant centroids for the labelled blobs
+    # # import random as rng  # for random colors of blobs
+    # rng.seed(53467)
 
-    # import matplotlib.pyplot as plt
-    # plt.imshow(d2)
-    # plt.show()
-    # mvt.idisp(d2)
-    im2 = Image('images/multiblobs_edgecase.png')
-    im2.disp()
+    # drawing = np.zeros((im.shape[0], im.shape[1], 3), dtype=np.uint8)
+    # colors = [None]*len(b)
+    # icont = [None]*len(b)
+    # for i in range(len(b)):
+    #     icont[i] = i
+    #     colors[i] = (rng.randint(0, 256), rng.randint(
+    #         0, 256), rng.randint(0, 256))
+
+    #     cv.rectangle(drawing, (b[i].umin, b[i].vmin), (b[i].umax, b[i].vmax),
+    #                  colors[i], thickness=2)
+    #     # cv.putText(drawing, str(i), (int(b[i].uc), int(b[i].vc)),
+    #     #           fontFace=cv.FONT_HERSHEY_SIMPLEX, fontScale=1,
+    #     #           color=colors, thickness=2)
+
+    # drawing = b.drawBlobs(im, drawing, icont, colors,
+    #                       contourthickness=cv.FILLED)
+    # # mvt.idisp(drawing)
+
+    # # import matplotlib.pyplot as plt
+    # # plt.imshow(d2)
+    # # plt.show()
+    # # mvt.idisp(d2)
+    # im2 = Image('images/multiblobs_edgecase.png')
+    # im2.disp()
 
     # press Ctrl+D to exit and close the image at the end
     # import code
